@@ -14,6 +14,15 @@ class ComplexAverageCrossEntropy(Loss):
         return (real_loss + imag_loss) / 2.
 
 
+class ComplexAverageCrossEntropyIgnoreUnlabeled(ComplexAverageCrossEntropy):
+
+    def call(self, y_true, y_pred):
+        mask = tf.reduce_any(tf.cast(y_true, tf.bool), axis=-1)
+        y_true = tf.boolean_mask(y_true, mask)
+        y_pred = tf.boolean_mask(y_pred, mask)
+        return super(ComplexAverageCrossEntropyIgnoreUnlabeled, self).call(y_true, y_pred)
+
+
 class ComplexMeanSquareError(Loss):
 
     def call(self, y_true, y_pred):
@@ -40,6 +49,19 @@ class ComplexWeightedAverageCrossEntropy(ComplexAverageCrossEntropy):
         return weighted_losses
 
 
+class ComplexWeightedAverageCrossEntropyIgnoreUnlabeled(ComplexAverageCrossEntropy):
+
+    def __init__(self, weights, **kwargs):
+        self.class_weights = weights
+        super(ComplexWeightedAverageCrossEntropyIgnoreUnlabeled, self).__init__(**kwargs)
+
+    def call(self, y_true, y_pred):
+        mask = tf.reduce_any(tf.cast(y_true, tf.bool), axis=-1)
+        y_true = tf.boolean_mask(y_true, mask)
+        y_pred = tf.boolean_mask(y_pred, mask)
+        return super(ComplexWeightedAverageCrossEntropyIgnoreUnlabeled, self).call(y_true, y_pred)
+
+
 if __name__ == "__main__":
     import numpy as np
     y_true = np.random.randint(0, 2, size=(2, 3)).astype("float32")
